/*
    BlockGame for J2SE
    Copyright (C) 2003 Irene Cummings (i.n.c@angelfire.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;

import java.util.*;
import java.io.*;

import inc.games.*;

public class BlockGame extends JFrame implements MouseListener, KeyListener, ActionListener
{
	private static int m_Size=300;
	private Container m_ContentPane;
	private GameArea m_GameArea;
	private TextDisplay m_HSDisplay;
	private JTextField m_EnterName;

	public BlockGame()
	{
		m_ContentPane=this.getContentPane();

		//HS display
		m_HSDisplay=new TextDisplay(m_Size);
		m_HSDisplay.setText("kkkkk");
		//m_HSDisplay.setVisible(false);
		m_HSDisplay.addMouseListener(this);
		m_HSDisplay.addKeyListener(this);
		//m_ContentPane.add(m_HSDisplay);


		//Enter name
		m_EnterName=new JTextField();
		//m_EnterName.setVisible(false);
		m_EnterName.addActionListener(this);
		//m_ContentPane.add(m_EnterName);

		//game area 
		//(add wanted initial screen last or will not see it!)
		m_GameArea=new GameArea(m_Size,30,6);
		//m_GameArea.setVisible(true);
		m_GameArea.addMouseListener(this);
		m_GameArea.addKeyListener(this);
		m_ContentPane.add(m_GameArea);
	}

	public static void main(String [] args)
	{
		BlockGame myGame=new BlockGame();
		myGame.setSize(m_Size,m_Size);
		myGame.setVisible(true);
	}

	public void mouseClicked(MouseEvent event)
	{
		System.out.println("mouse clicked");
		Object obj=event.getSource();
		if (obj instanceof GameArea)
		{
			m_GameArea.requestFocusInWindow();
			GameArea game=(GameArea)obj;
			int x=event.getX();
			int y=event.getY();
			game.Respond2Event(x,y);
		}
		else if (obj instanceof TextDisplay)
		{
			m_HSDisplay.requestFocusInWindow();
		}
	}

	//currently unused
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e){}

	public void keyTyped(KeyEvent event)
	{
		System.out.println("key typed");
		Object obj=event.getSource();
		char key=event.getKeyChar();
		if (obj instanceof GameArea)
		{
			GameArea game=(GameArea)obj;
			if (key=='s')
			{
				System.out.println("key s");
				String hsText=m_GameArea.GetHighScore();
				m_HSDisplay.setText(hsText);
				int winner=m_GameArea.IsScoreGood();
				m_GameArea.setVisible(false);
				if (winner>0)
				{
					System.out.println("enter name");
					m_ContentPane.remove(m_GameArea);
					m_ContentPane.add(m_EnterName);
					m_EnterName.setVisible(true);
				}
				else
				{
					System.out.println("show scores");
					m_ContentPane.remove(m_GameArea);
					m_ContentPane.add(m_HSDisplay);
					m_HSDisplay.setVisible(true);
				}
			}
			else if (key=='q')
			{
				System.out.println("key q");
				System.exit(0);
			}
			else if (key=='n')
			{
				System.out.println("key n");
				game.NewGame(m_Size);
			}
			else
			{
				System.out.println("Unknown key stroke!");
				System.out.println("\ts - Save score");
				System.out.println("\tq -  Quit");
				System.out.println("\tn - New game");
			}
		}
		else if (obj instanceof TextDisplay)
		{
			if (key=='q')
			{
				System.out.println("key q");
				m_HSDisplay.setVisible(false);
				m_ContentPane.remove(m_HSDisplay);
				m_ContentPane.add(m_GameArea);
				m_GameArea.setVisible(true);
			}
			else
			{
				System.out.println("Unknown key stroke!");
				System.out.println("\tq - Return to main screen");
			}
			
		}
	}

	//unused
	public void keyPressed(KeyEvent e) {};
	public void keyReleased(KeyEvent e) {};

	public void actionPerformed(ActionEvent event)
	{
		System.out.println("action performed");
		if (event.getSource() instanceof JTextField)
		{
			JTextField field=(JTextField)event.getSource();
			String text=field.getText();
			field.setText("");
			m_GameArea.SaveScore(text);
			m_EnterName.setVisible(false);
			m_ContentPane.remove(m_EnterName);
			m_ContentPane.add(m_HSDisplay);
			m_HSDisplay.setVisible(true);
			System.out.println("focusable="+this.isFocusable()+" displayable="+this.isDisplayable()+" visiable="+this.isVisible()+" enabled="+this.isEnabled());
			m_HSDisplay.requestFocusInWindow();
			System.out.println("showing="+this.isShowing()+" lightweight="+this.isLightweight());
			System.out.println("display hs='"+m_HSDisplay.m_Text+"'");
		}
	}

}

class GameArea extends JPanel implements AbstractCanvas
{
	private BlockGameBase m_BlockGame;
	private HighScore m_HighScores;

	public GameArea(int sizeScreen,int sizeBlock, int numbColours)
	{
		m_BlockGame=new BlockGameBase(this,sizeScreen,sizeScreen,sizeBlock,numbColours);
		m_HighScores=new HighScore();
	}

	public void NewGame(int sizeScreen)
	{
		int sizeBlock=m_BlockGame.m_BlockSize;
		int numbColours=m_BlockGame.m_ColourNo;
		m_BlockGame=new BlockGameBase(this,sizeScreen,sizeScreen,sizeBlock,numbColours);
	}

	protected void paintComponent(Graphics graphic)
	{
		m_BlockGame.Paint(new GraphicJ2SE(graphic));
	}

	public void Repaint()
	{
		repaint();
	}

	public void Respond2Event(int x, int y)
	{
		m_BlockGame.Respond2Event(x,y);
	}

	//thin wrappers around data base
	public void SaveScore(String name)
	{
		m_HighScores.SaveScore(name,m_BlockGame.GetHighScore());
	}

	public String GetHighScore()
	{
		return m_HighScores.GetHighScore(m_BlockGame.GetHighScore());
	}

	public int IsScoreGood()
	{
		return m_HighScores.IsScoreGood(m_BlockGame.GetHighScore());
	}

}


//Amongst other things writes score records to file and reads them back
//Each record is of a set length...
class HighScore extends HighScoreBase
{
	private HSRecord[] m_HighScores=new HSRecord[10];
	//private HSRecord[] m_HighScores={null,null,null,null,null,null,null,null,null,null};
	private int m_NoSavedScores=0;
	private File m_HighScoreDB=new File("HighScores.db");

	public HighScore()
	{
		try
		{
			//open HS table and store contents
			FileInputStream db=new FileInputStream(m_HighScoreDB);
			int count=0;
			int offset=0;
			while (offset<m_HighScoreDB.length())
			{
				offset=count*m_RecordLength;
				byte[] record=new byte[m_RecordLength];
				db.read(record,offset,m_RecordLength);
				int[] scoreA={0};
				StringBuffer nickName=new StringBuffer();
				long[] date={0};
				this.DecomposeRecord(record,scoreA,nickName,date);
				String name=nickName.toString();
				name.trim();	//rm padding (if any)
				m_HighScores[count]=new HSRecord(count);
				m_HighScores[count].SetScore(scoreA[0]);
				m_HighScores[count].SetDate(date[0]);
				m_HighScores[count].SetName(name);
				count++;
			}
			db.close();
			m_NoSavedScores=count;
			Arrays.sort(m_HighScores,0,m_NoSavedScores);
		}
		catch (IOException e)
		{
			//file does not exist
		}
	}

	//write record to file at offset
	protected void WriteRecord(byte[] rec,int offset)
	{
		try
		{
			FileOutputStream db=new FileOutputStream(m_HighScoreDB);
			db.write(rec,offset,rec.length);
			db.close();
		}
		catch (IOException e)
		{
			//file does not exist & can not be created
		}
	}

	//Highscore Functions

	//returns whether usr has made the hs table
	//-1 too low, 0 equal, 1 good enough to add, 2 table not full (-2 error)
	public int IsScoreGood(int score)
	{
		String errStr=new String("IsScoreGood(");
		errStr+=score;
		errStr+=") and m_NoSavedScores=";
		errStr+=m_NoSavedScores;
		System.out.println(errStr);
		int rc=-1;
		if (m_NoSavedScores<10)
		{
			rc=2;
		}
		else if (score==m_HighScores[0].GetScore())
		{
			rc=0;
		}
		else if (score>m_HighScores[0].GetScore())
		{
			rc=1;
		}
		return rc;
	}

	//add HS to database
	public void SaveScore(String nickName,int score)
	{
		int scoreEnum=this.IsScoreGood(score);
		if (scoreEnum>0)
		{
			long date=System.currentTimeMillis();

			byte asByte[]=this.CreateRecord(score,nickName,date);
			//System.out.println(new String(asByte));
			//get records sorted and if lowest one smaller replace it!
			int saveLoc=-1;
			if (scoreEnum==2)
			{
				saveLoc=m_NoSavedScores;
				m_NoSavedScores++;
			}
			else
			{
				saveLoc=m_HighScores[0].GetID();
			}
				
			m_HighScores[saveLoc]=new HSRecord(saveLoc);
			m_HighScores[saveLoc].SetScore(score);
			m_HighScores[saveLoc].SetDate(date);
			m_HighScores[saveLoc].SetName(nickName);
			Arrays.sort(m_HighScores,0,m_NoSavedScores);
			this.WriteRecord(asByte,saveLoc*m_RecordLength);
		}
		else
		{
			//System.out.println("Trying to save too low score ");
		}
	}

	//return HS display string
	public String GetHighScore(int score)
	{
		String result=new String("Current:\n");
		result+=score+"\n"+"Previous:\n";

		for (int i=0;i<m_NoSavedScores;i++)
		{
			result+=m_HighScores[i].GetScore();
			result+="\t";
			StringBuffer dateStr=this.LongDateToString(m_HighScores[i].GetDate());
			result+=dateStr;
			result+="\t";
			result=m_HighScores[i].GetName();
		}
		return result;
	}
}

class HSRecord implements Comparable
{
	private int m_Score=-1;
	private long m_Date=-1;
	private String m_Name;
	private int m_RecordID=-1;

	public HSRecord(int id) {m_RecordID=id;};

	public void SetScore(int score) {m_Score=score;};

	public void SetDate(long date) {m_Date=date;};

	public void SetName(String name) {m_Name=name;};

	public int GetScore() {return m_Score;};

	public int GetID() {return m_RecordID;};

	public String GetName() {return m_Name;};

	public long GetDate() {return m_Date;};

	public int compareTo(Object arg)
	{
		if (arg==null)
		{
			return 1;
		}
		HSRecord comp=(HSRecord )arg;
		if (comp.m_Score==m_Score)
		{
			return 0;
		}
		else if (comp.m_Score>m_Score)
		{
			return -1;
		}
		else
		{
			return 1;
		}
	}

}


class TextDisplay extends JPanel
{
	public String m_Text;

	public TextDisplay(int sizeScreen)
	{
		this.setPreferredSize(new Dimension(sizeScreen,sizeScreen));
		this.setBackground(new Color(250));
		m_Text=new String();
	}

	protected void paintComponent(Graphics graphic)
	{
		System.out.println("paintComponent TextDisplay="+m_Text);
		super.paintComponent(graphic);
		graphic.drawString(m_Text,50,50);
	}

	public void setText(String text)
	{
		System.out.println("setText("+text+")");
		m_Text=text;
	}

	//tmp
	public void repaint(String str)
	{
		System.out.println("repaint("+str+")");
		this.repaint();
		this.paintImmediately(0,0,300,300);
	}

	public void paint(Graphics graphic)
	{
		System.out.println("paint");
		super.paint(graphic);
	}
}



